import org.junit.Assert;
import org.junit.Test;
import org.kie.api.KieBase;
import org.kie.api.KieServices;
import org.kie.api.builder.KieBuilder;
import org.kie.api.builder.KieFileSystem;
import org.kie.api.builder.KieModule;
import org.kie.api.builder.KieRepository;
import org.kie.api.builder.Message;
import org.kie.api.builder.ReleaseId;
import org.kie.api.builder.model.KieModuleModel;
import org.kie.api.runtime.KieContainer;
import org.kie.api.runtime.KieSession;

import java.util.Date;
import java.util.GregorianCalendar;

public class ForallWithNotEqualConstraintTest {

    public static final String KIEBASE = "forall-with-not-equal-on-dates";

    public static final String PACKAGE = "org.drools.reproducer";

    private static final String DRL = "package " + PACKAGE +
            "\n" +
            "rule \"forall with not equal\"\n" +
            "when forall(java.util.Date(this != \"29-Dec-2019\"))\n" +
            "then\n" +
            "end\n";


    @Test
    public void buildKieBase() {

        System.out.println(DRL);

        KieServices ks = KieServices.Factory.get();
        KieRepository kr = ks.getRepository();

        KieModule kModule = buildKieModule("0.0.1", DRL);
        KieContainer kContainer = ks.newKieContainer(kModule.getReleaseId());
        Assert.assertNotNull(kContainer);
        kr.removeKieModule(kModule.getReleaseId());

        KieBase kieBase = kContainer.getKieBase(KIEBASE);

        Assert.assertNotNull(kieBase);

        KieSession kieSession = kieBase.newKieSession();

        kieSession.insert(new GregorianCalendar(2020, 4, 6).getTime());
        kieSession.insert(new GregorianCalendar(2020, 4, 5).getTime());

        int n = kieSession.fireAllRules();

        Assert.assertEquals(1, n);
    }

    private static KieModule buildKieModule(String version, String rules) {
        KieServices ks = KieServices.Factory.get();
        KieFileSystem kfs = ks.newKieFileSystem();

        ReleaseId rid = ks.newReleaseId(PACKAGE, KIEBASE, version);
        kfs.generateAndWritePomXML(rid);

        KieModuleModel kModuleModel = ks.newKieModuleModel();
        kModuleModel.newKieBaseModel(KIEBASE)
                .addPackage(PACKAGE);

        kfs.writeKModuleXML(kModuleModel.toXML());

        kfs.write("src/main/resources/org/drools/reproducer/rules.drl", rules);

        KieBuilder kb = ks.newKieBuilder(kfs);
        kb.buildAll(); // kieModule is automatically deployed to KieRepository if successfully built.
        if (kb.getResults().hasMessages(Message.Level.ERROR)) {
            throw new RuntimeException("Build Errors:\n" + kb.getResults().toString());
        }

        return kb.getKieModule();
    }

}
